from .handler.indigo import Indigo
from .handler.spice_jet import SpiceJet


class Airlines(object):
    def __init__(self, aviation, config):
        self.aviation = None
        self.mapping = {
            "indigo": Indigo,
            "spice_jet": SpiceJet
        }
        self.setup(aviation, config)

    def setup(self, aviation, config):
        try:
            self.aviation = self.mapping[aviation](config)
        except KeyError as e:
            raise Exception("This aviation company is not yet available, Please refer docs!")

    def get_pnr_status(self, parameters):
        return self.aviation.get_pnr_status(parameters)




from flight_status.handler import BeautifulSoup


class Scraper(object):
    def __init__(self):
        self.error_message = "Error: Not able to fetch details"

    def get_pnr_status(self, page_source):

        soup = BeautifulSoup(page_source, features="html.parser")
        summary = soup.find(name="div", attrs={"class": "conf-summary"})
        details = summary.find("div", {"class": "trip-summary-id"})

        try:
            ticket_status = soup.find("div", {"class": "cnf-summary-status"}).text
        except Exception as err:
            ticket_status = self.error_message

        destinations = []
        try:
            for item in details.find_all("p"):
                destinations.append(item.find("span").text)
        except Exception as err:
            destinations = None

        flight_summary = []
        try:
            for each in details.find("ul", {"class": "conf-flight-details"}).find_all("li"):
                flight_summary.append(each.text.strip())
        except Exception as err:
            flight_summary = None

        passengers = []
        try:
            passenger_table = soup.find("table", {"id": "view-itinerary-hbo"})
            for tr in passenger_table.find("tbody").find_all('tr'):
                passengers.append({
                    'Name': tr.find_all('td')[0].text.strip(),
                    'seat': tr.find_all('td')[1].text.strip()
                })
        except Exception as err:
            passengers = []

        try:
            flight_details = {
                "from_airport": destinations[0],
                "to_airport": destinations[1],
                "direction": flight_summary[0],
                "date_of_journey": flight_summary[1],
                "num_of_travellers": len(passengers)
            }
        except Exception as err:
            flight_details = self.error_message

        response = {
            "flight_details": flight_details,
            "passenger_details": passengers,
            "status": ticket_status
        }

        return response



from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium.common.exceptions import TimeoutException


class WebController(object):
    def __init__(self, user_config):
        self.driver = self.setup(user_config)
        self.max_delay_timeout = 25

    def setup(self, user_config):
        # instantiate a chrome options object so you can set the size and headless preference
        options = Options()
        options.add_argument("--headless")
        driver_supported = {
            "chrome": webdriver.Chrome
        }
        if {"webdriver", "executable"}.issubset(user_config):
            if user_config["webdriver"] not in driver_supported.keys() or not user_config["executable"]:
                raise Exception("Either this webdriver not supported now or executable is missing!")
            driver = driver_supported[user_config["webdriver"]](options=options,
                                                                executable_path=user_config["executable"])
            return driver
        else:
            raise Exception("Please check the webdriver config provided, refer docs.")

    def open_page(self, url):
        self.driver.get(url)

    def get_element(self, tag_type, name):
        element = None
        if tag_type == "id":
            element = self.driver.find_element_by_id(name)
        elif tag_type == "class":
            element = self.driver.find_element_by_class_name(name)
        return element

    def input_text(self, tag_type, name, value):
        input_element = self.get_element(tag_type, name)
        input_element.send_keys(value)

    def click_element(self, tag_type, name):
        element = self.get_element(tag_type, name)
        self.driver.execute_script("arguments[0].click();", element)

    def get_attribute(self, tag_type, name, attribute):
        element = self.get_element(tag_type, name)
        return element.get_attribute(attribute)

    def wait_till_load(self, tag_type, name):
        BY = {
            "class": By.CLASS_NAME,
            "id": By.ID
        }
        try:
            WebDriverWait(self.driver, self.max_delay_timeout).until(
                EC.presence_of_element_located((BY[tag_type], name)))
            return True

        except TimeoutException:
            print("Loading took too much time!")
            return False

    def exit_driver(self):
        self.driver.quit()



from flight_status.handler.web_controller import WebController
from flight_status.handler.spice_jet.scraper import Scraper


class SpiceJet(object):
    def __init__(self, config):
        self.url = {
            "home": "https://book.spicejet.com/",
            "pnr_status": "https://book.spicejet.com/RetrieveBooking.aspx"
        }
        self.user_config = config

    def get_pnr_status(self, parameters):
        pass
        web = WebController(self.user_config)
        web.open_page(url=self.url["pnr_status"])
        web.input_text(tag_type="id",
                       name="ControlGroupRetrieveBookingView_BookingRetrieveInputRetrieveBookingView_ConfirmationNumber",
                       value=parameters["booking_reference"])
        web.input_text(tag_type="id",
                       name="ControlGroupRetrieveBookingView_ResendItineraryRetrieveBookingView_RecordLocator",
                       value=parameters["booking_reference"])
        web.input_text(tag_type="id",
                       name="ControlGroupRetrieveBookingView_BookingRetrieveInputRetrieveBookingView_CONTACTEMAIL1",
                       value=parameters["email_lastname"])
        web.click_element(tag_type="id",
                          name="ControlGroupRetrieveBookingView_BookingRetrieveInputRetrieveBookingView_ButtonRetrieve")
        status = web.wait_till_load(tag_type="id", name="bookingDetail")

        if status:
            html_source = web.get_attribute(tag_type="id", name="itineraryBody", attribute="innerHTML")
            web.exit_driver()
            scraper_controller = Scraper()
            response = scraper_controller.get_pnr_status(html_source)
            return response
        else:
            return {"error": "Please check the input details / Server Error"}



from setuptools import setup, find_packages

with open('README.md') as readme_file:
    README = readme_file.read()

setup(
    name='flight-status',
    version='1.0.0',
    description='PIP package to get status of flight pnr status and other details [Indian Airlines]',
    long_description_content_type="text/markdown",
    long_description=README,
    license='MIT',
    packages=find_packages(),
    author='Aman Tiwari',
    author_email='tiwari.aman85@gmail.com',
    keywords=['Flight', 'PNR', 'Pnr status', 'India', 'Airlines'],
    url='https://github.com/tiwari85aman/flight-status.git',
    download_url='https://pypi.org/project/flight-status/',
    python_requires=">=2.7.0",
    install_requires=[
        'beautifulsoup4<=4.9.1',
        'selenium<=3.141.0',
    ],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
    ],

)



from flight_status.handler import BeautifulSoup


class Scraper(object):
    def __init__(self):
        self.error_message = "Error: Not able to fetch details"

    def get_pnr_status(self, page_source):
        soup = BeautifulSoup(page_source, features="html.parser")
        try:
            ticket_status = soup.find(name="td", attrs={"class": "width-pnr-status"}).find("strong").text
        except Exception as err:
            ticket_status = self.error_message

        passengers = []
        try:
            for tr in soup.find("tbody", {"id": "passengerBody"}).find_all("tr", {"class": 'passenger-info-border'},
                                                                           recursive=False):
                if tr.find("td"):
                    passengers.append({
                        'Name': tr.find_all('td')[0].text.strip(),
                        'Pax Type': [each.strip() for each in tr.find_all('td')[1].text.replace("\n", "").split(",")],
                    })
        except Exception as err:
            passengers = []

        temp = {}
        try:
            for tr in soup.find("table", {"id": "flight-journey-detail"}).find("tbody").find_all('tr'):
                if tr.find("td"):
                    temp = {
                        'doj': tr.find_all('td')[0].text.strip(),
                        'Flight': tr.find_all('td')[1].text.strip(),
                        'From': tr.find_all('td')[2].text.strip(),
                        'To': tr.find_all('td')[3].text.strip(),
                        'Dept': tr.find_all('td')[4].text.strip(),
                        'Arr': tr.find_all('td')[5].text.strip(),
                        'Current': tr.find_all('td')[6].text.strip(),
                    }
        except Exception as err:
            temp = None
        try:
            flight_details = {
                "from_airport": temp["From"],
                "to_airport": temp["To"],
                "date_of_journey": temp["doj"],
                "num_of_travellers": len(passengers)
            }
        except Exception as err:
            flight_details = self.error_message

        response = {
            "flight_details": flight_details,
            "passenger_details": passengers,
            "status": ticket_status
        }

        return response
